.multiline-table-value {
    background-color: #f0f0f0;
}

.multiline-table-value:hover {
    cursor: pointer;
    background-color: #ffcc00;
}

.origin-only-row {
    background-color: rgba(
            255,
            0,
            0,
            0.3
    );
}

.destination-only-row {
    background-color: rgba(0, 128, 0, 0.3);
}