.singleCellLayout {
    display: flex;
    flex-direction: row;
    /*justify-content: space-between;*/
    align-items: flex-start; /* Align items to the start of the cross axis (top) */
}

.executionOrder {
    display: flex;
    /*margin-right: 4px;*/
    margin-left: 10px;
    padding: 0;
    /*width: 40px;*/
    white-space: pre;
    word-break: break-all;
    width: 30px;
}

.cell-code {
    border: 2px solid #dedede;
    padding: 0.5em;
    background-color: #f3f3f3;
    .ace_hidden-cursors {
        opacity: 0;
    }
    .ace_bracket { display: none }
}

.type_code{
    width: 90%;
}

.type_markdown{
    width: 93%;
}

.add-row-marker {
    background: rgba(0, 128, 0, 0.3);
    position: absolute;
}

.delete-row-marker {
    background: rgba(
            255,
            0,
            0,
            0.3
    ); /* red */
    position: absolute;
}

.ace_editor {
    position: relative;
    overflow: hidden;
    padding: 0;
    font: 13px/normal 'Menlo', 'Ubuntu Mono', 'Consolas', 'Source Code Pro', 'source-code-pro', monospace;
    font-size: 13px;
    direction: ltr;
    text-align: left;
    -webkit-tap-highlight-color: rgba(0, 0, 0, 0);
}
