.variable-row{
    display: flex;
    flex-direction: row;
    white-space: nowrap;      /* Prevents wrapping of text onto a new line */
    overflow: hidden;         /* Hides any overflow that doesn't fit in the container */
    text-overflow: ellipsis;  /* Optionally, adds an ellipsis to indicate text has been truncated */
    padding: 0.5em;
    /*border: 1px solid #1e1f22;*/
}

.parent-var-row{
    flex:1;
    overflow: hidden;
    text-overflow: ellipsis;
}

.origin-only-row {
    background-color: rgba(
            255,
            0,
            0,
            0.3
    );
}

.destination-only-row {
    background-color: rgba(0, 128, 0, 0.3);
}

.add-watch-input{
    padding-left: 1em;
    font-size: 1em;
    width: 90%;
    height: 2em;
    border: none;
    background-color: transparent;
}

.add-watch{
    border:1px solid #b0a7a7;
    width: 100%;
    display: flex;
    flex-direction: row;
    justify-content: space-between;
}