/*
 * @Author: University of Illinois at Urbana Champaign
 * @Date: 2023-07-15 22:04:04
 * @LastEditTime: 2023-07-29 10:49:59
 * @FilePath: /src/util/Cell.ts
 * @Description:
 */
export interface Cell {
  content: string;
  execNum?: string;
  type: string;
}
