*,
*:before,
*:after {
    box-sizing: border-box;
    outline: none;
}

html,
body {
    height: 100%;
    width: 100%;
    margin: 0;
    padding: 0;

    color: rgb(0, 0, 0);
    /*font-family: -apple-system, BlinkMacSystemFont, "Segoe UI", "Helvetica Neue",*/
    /*Helvetica, Arial, sans-serif, "Apple Color Emoji", "Segoe UI Emoji",*/
    /*"Segoe UI Symbol";*/
    /*font-family: 'Fira Sans', sans-serif;*/
    background: #fafbfb;
}

body {
    -webkit-font-smoothing: antialiased;
    -moz-osx-font-smoothing: grayscale;
}

#root {
    margin: 0 auto;
    height: 100%;
    width: 100%;
}
